.user-list .pf-c-table {
  margin-bottom: 2rem;
}

.user-list .control-label {
  padding-top: 0.375rem;
}

.user-list th {
  padding-top: 0.375rem;
}

.form-customizations .col-sm-2,
.form-customizations .col-sm-10,
.form-customizations .col-sm-12 {
  padding-left: 0;
  padding-right: 0;
}

.form-customizations .form-horizontal .form-group {
  margin-left: 0;
  margin-right: 0;
}
